@@ -1,9 +1,9 @@
-# Version 1.6-1
+# Version 1.8.1-1
 
 import re
 import sys
 from hostlist import expand_hostlist, collect_hostlist
-import commands
+import subprocess
 
 # Exceptions
 
@@ -59,7 +59,7 @@ class Node(Entity):
         return "<Node: %s>" % (self.name)
 
     def expand_to_node_set(self):
-        return set([self])
+        return {self}
 
     def add_to_group(self, group):
         if group.level in self.part_of:
@@ -74,6 +74,9 @@ class Node(Entity):
     def get_group(self, level):
         return self.part_of.get(level)
 
+    def get_clustershell_name(self):
+        return self.name
+
 class Group(Entity):
     def __init__(self, hierarchy, level, name, all_nodes_group = False):
         self.hierarchy = hierarchy
@@ -83,9 +86,10 @@ class Group(Entity):
         self.representative = None
         self.sorted_nodes = None
         self.all_nodes_group = all_nodes_group
+        self.nodespecs = [] # Raw list of all nodespecs added
 
     def __repr__(self):
-        return "<Group %s: %s>" % (self.level, self.name)
+        return f"<Group {self.level}: {self.name}>"
 
     def expand_to_node_set(self):
         return self.node_set
@@ -105,6 +109,7 @@ class Group(Entity):
 
     def add_nodespec(self, nodespec):
         #print "  NODES", self.name, "-->", nodespec
+        self.nodespecs.append(nodespec)
         node_set = set()
         for name in expand_hostlist(nodespec):
             entity = self.hierarchy.get_entity(name, auto_add_node = self.all_nodes_group)
@@ -116,6 +121,21 @@ class Group(Entity):
             node.add_to_group(self)
         self.representative = None
 
+    def get_clustershell_name(self):
+        return f"@{self.level}:{self.name}"
+
+    def get_clustershell_definition(self):
+        res = []
+        for ns in self.nodespecs:
+            converted_set = set()
+            for e in set(expand_hostlist(ns)):
+                entity = self.hierarchy.get_entity(e)
+                converted_set.add(entity.get_clustershell_name())
+            part = collect_hostlist(converted_set)
+
+            res.append(part)
+        return ",".join(res)
+
 class FakedNode(Entity):
     """Use to fake nodes with indexes for --index"""
     def __init__(self, nodename, index, separator):
@@ -126,13 +146,21 @@ class Hierarchy:
         self.names = {} # name -> Node or Group
         self.level_names = set() # level names used for abbreviation lookup
 
+        self.level_names_list = [] # level names in the order mentioned in the file
+        self.level_groups = {} # level name -> list of Groups
+
     def create_group(self, level, name, all_nodes_group = False):
         #print "GROUP", level, name
+        if not level in self.level_names:
+            self.level_names_list.append(level)
+            self.level_groups[level] = []
         self.level_names.add(level)
+
         if name in self.names:
             raise DuplicatedGroup("group %s seen twice" % name)
         group = Group(self, level, name, all_nodes_group)
         self.names[name] = group
+        self.level_groups[level].append(group)
         return group
 
     def create_or_get_group(self, level, name):
@@ -150,7 +178,7 @@ class Hierarchy:
         return self.names[name]
 
     def parse_file(self, file_or_filename):
-        if type(file_or_filename) == type(""):
+        if isinstance(file_or_filename, str):
             f = open(file_or_filename)
         else:
             f = file_or_filename
@@ -168,11 +196,11 @@ class Hierarchy:
                 if rest:
                     current_group.add_nodespec(rest)
                 continue
-            
+
             # <level> <name>: <parts>
             m = re.match(r'^([a-z]+)\s+([a-z0-9]+):\s*(.*)', line)
             if m: 
-                (level, name, rest) = m.group(1,2,3)
+                (level, name, rest) = m.group(1, 2, 3)
                 current_group = self.create_group(level, name)
                 if rest:
                     current_group.add_nodespec(rest)
@@ -190,13 +218,13 @@ class Hierarchy:
             raise BadConfigSyntax(line)
 
     def parse_slurm(self):
-        rc, slurm_data = commands.getstatusoutput('/usr/bin/squeue -aho "%i %u %N"')
-        if rc <> 0:
+        rc, slurm_data = subprocess.getstatusoutput('/usr/bin/squeue -aho "%i %u %N"')
+        if rc != 0:
             raise FailedSLURM("Failed to get data from SLURM")
 
         for line in slurm_data.split("\n"):
             fields = line.split()
-            if len(fields) <> 3:
+            if len(fields) != 3:
                 continue
             job, user, nodes = fields
 
@@ -213,7 +241,7 @@ class Hierarchy:
         elif len(matching) == 0:
             raise UnknownLevel("unknown level %s" % level)
         else:
-            raise AmbiguousLevel("ambiguous level %s matching %s" % (level,
+            raise AmbiguousLevel("ambiguous level {} matching {}".format(level,
                                                                      ", ".join(matching)))
 
     def expand_to_node_set(self, nodespec):
@@ -334,3 +362,13 @@ class Hierarchy:
                                  (level,
                                   collect_name_hostlist(missing)))
         return res
+
+    # Convert to ClusterShell YAML definition
+    def convert_to_clustershell(self):
+        for level in self.level_names_list:
+            if level == "all": continue
+            print("%s:" % level)
+            for group in self.level_groups[level]:
+                print(f"   {group.name}: '{group.get_clustershell_definition()}'")
+            print()
+
