@@ -1,12 +1,12 @@
 import nodes
 import unittest
-import cStringIO
+import io
 
 class Test1(unittest.TestCase):
 
     def setUp(self):
         self.h = nodes.Hierarchy()
-        f = cStringIO.StringIO("""
+        f = io.StringIO("""
 nodes: n[1-99]
 enclosure e1: n[1-4]
 enclosure e2: n[5-8]
@@ -20,11 +20,11 @@ oddball o1: n[6-10]
         f.close()
 
     def expands_to(self, nodespec, hostlist):
-        self.assertEqual(self.h.expand(nodespec), hostlist)
+        self.assertEqual(nodes.collect_name_hostlist(self.h.to_nodes(nodespec)), hostlist)
         
     def expands_bad(self, nodespec):
         self.assertRaises(nodes.NodesException,
-                          self.h.expand,  nodespec)
+                          self.h.to_nodes,  nodespec)
 
     def test_expand(self):
         self.expands_to("",       "")
@@ -50,25 +50,25 @@ oddball o1: n[6-10]
 
     def ufrg(self, level, nodespec, up, fill, representative, gather):
         if up is not None:
-            self.assertEqual(self.h.up(level, nodespec), up)
+            self.assertEqual(nodes.collect_name_hostlist(self.h.up(level, nodespec)), up)
         else:
             self.assertRaises(nodes.NodesException,
                               self.h.up, level, nodespec)
 
         if fill is not None:
-            self.assertEqual(self.h.fill(level, nodespec), fill)
+            self.assertEqual(nodes.collect_name_hostlist(self.h.fill(level, nodespec)), fill)
         else:
             self.assertRaises(nodes.NodesException,
                               self.h.fill, level, nodespec)
 
         if representative is not None:
-            self.assertEqual(self.h.representative(level, nodespec), representative)
+            self.assertEqual(nodes.collect_name_hostlist(self.h.representative(level, nodespec)), representative)
         else:
             self.assertRaises(nodes.NodesException,
                               self.h.representative, level, nodespec)
 
         if gather is not None:
-            self.assertEqual(self.h.gather(level, nodespec), gather)
+            self.assertEqual(nodes.collect_name_hostlist(self.h.gather(level, nodespec)), gather)
         else:
             self.assertRaises(nodes.NodesException,
                               self.h.gather, level, nodespec)
@@ -124,12 +124,12 @@ oddball o1: n[6-10]
 
     def um(self, level, nodespec, up, up_missing):
         if up is not None:
-            self.assertEqual(self.h.up(level, nodespec), up)
+            self.assertEqual(nodes.collect_name_hostlist(self.h.up(level, nodespec)), up)
         else:
             self.assertRaises(nodes.NodesException,
                               self.h.up, level, nodespec)
         if up_missing is not None:
-            self.assertEqual(self.h.up(level, nodespec, "M"), up_missing)
+            self.assertEqual(nodes.collect_name_hostlist(self.h.up(level, nodespec, "M")), up_missing)
         else:
             self.assertRaises(nodes.NodesException,
                               self.h.up, level, nodespec, "M")
